@@ -23,6 +23,7 @@ from netplan.cli.commands.try_command import NetplanTry
 from netplan.cli.commands.info import NetplanInfo
 from netplan.cli.commands.set import NetplanSet
 from netplan.cli.commands.get import NetplanGet
+from netplan.cli.commands.sriov_rebind import NetplanSriovRebind
 
 __all__ = [
     'NetplanApply',
@@ -33,4 +34,5 @@ __all__ = [
     'NetplanInfo',
     'NetplanSet',
     'NetplanGet',
+    'NetplanSriovRebind'
 ]
