@@ -18,13 +18,26 @@
 #pragma once
 
 #include "parse.h"
+#include <glib.h>
+#include <yaml.h>
 
 
 gboolean is_ip4_address(const char* address);
 gboolean is_ip6_address(const char* address);
+gboolean is_hostname(const char* hostname);
+gboolean validate_ovs_target(gboolean host_first, gchar* s);
+
+NETPLAN_ABI gboolean
+is_wireguard_key(const char* hostname);
 
 gboolean
-validate_netdef_grammar(net_definition* nd, yaml_node_t* node, GError** error);
+validate_netdef_grammar(const NetplanParser* npp, NetplanNetDefinition* nd, yaml_node_t* node, GError** error);
 
 gboolean
-validate_backend_rules(net_definition* nd, GError** error);
+validate_backend_rules(const NetplanParser* npp, NetplanNetDefinition* nd, GError** error);
+
+gboolean
+validate_sriov_rules(const NetplanParser* npp, NetplanNetDefinition* nd, GError** error);
+
+gboolean
+validate_default_route_consistency(const NetplanParser* npp, GHashTable* netdefs, GError** error);
