@@ -33,6 +33,11 @@ if shutil.which('python3-coverage'):
 
 # Make sure we can import our development netplan.
 os.environ.update({'PYTHONPATH': '.'})
+os.environ.update({'LD_LIBRARY_PATH': '.:{}'.format(os.environ.get('LD_LIBRARY_PATH'))})
+
+
+def _load_yaml(text):
+    return yaml.load(text, Loader=yaml.SafeLoader)
 
 
 class TestArgs(unittest.TestCase):
@@ -63,9 +68,11 @@ class TestGenerate(unittest.TestCase):
         self.workdir = tempfile.TemporaryDirectory()
 
     def test_no_config(self):
-        out = subprocess.check_output(exe_cli + ['generate', '--root-dir', self.workdir.name])
+        p = subprocess.Popen(exe_cli + ['generate', '--root-dir', self.workdir.name], stdout=subprocess.PIPE,
+                             stderr=subprocess.PIPE)
+        (out, err) = p.communicate()
         self.assertEqual(out, b'')
-        self.assertEqual(os.listdir(self.workdir.name), [])
+        self.assertEqual(os.listdir(self.workdir.name), ['run'])
 
     def test_with_empty_config(self):
         c = os.path.join(self.workdir.name, 'etc', 'netplan')
@@ -145,6 +152,7 @@ class TestGenerate(unittest.TestCase):
 
 
 class TestIfupdownMigrate(unittest.TestCase):
+
     def setUp(self):
         self.workdir = tempfile.TemporaryDirectory()
         self.ifaces_path = os.path.join(self.workdir.name, 'etc/network/interfaces')
@@ -203,20 +211,20 @@ source-directory /etc/network/interfaces.d''')[0]
 
     def test_dhcp4(self):
         out = self.do_test('auto en1\niface en1 inet dhcp')[0]
-        self.assertEqual(yaml.load(out), {'network': {
+        self.assertEqual(_load_yaml(out), {'network': {
             'version': 2,
             'ethernets': {'en1': {'dhcp4': True}}}}, out.decode())
 
     def test_dhcp6(self):
         out = self.do_test('auto en1\niface en1 inet6 dhcp')[0]
-        self.assertEqual(yaml.load(out), {'network': {
+        self.assertEqual(_load_yaml(out), {'network': {
             'version': 2,
             'ethernets': {'en1': {'dhcp6': True}}}}, out.decode())
 
     def test_dhcp4_and_6(self):
         out = self.do_test('auto lo\niface lo inet loopback\n\n'
                            'auto en1\niface en1 inet dhcp\niface en1 inet6 dhcp')[0]
-        self.assertEqual(yaml.load(out), {'network': {
+        self.assertEqual(_load_yaml(out), {'network': {
             'version': 2,
             'ethernets': {'en1': {'dhcp4': True, 'dhcp6': True}}}}, out.decode())
 
@@ -224,7 +232,7 @@ source-directory /etc/network/interfaces.d''')[0]
         out = self.do_test('iface lo inet loopback\nauto lo\nsource-directory interfaces.d',
                            dropins={'interfaces.d/std': 'auto en1\niface en1 inet dhcp',
                                     'interfaces.d/std.bak': 'some_bogus dontreadme'})[0]
-        self.assertEqual(yaml.load(out), {'network': {
+        self.assertEqual(_load_yaml(out), {'network': {
             'version': 2,
             'ethernets': {'en1': {'dhcp4': True}}}}, out.decode())
 
@@ -232,7 +240,7 @@ source-directory /etc/network/interfaces.d''')[0]
         out = self.do_test('iface lo inet loopback\nauto lo\nsource-directory /etc/network/defs/my',
                            dropins={'defs/my/std': 'auto en1\niface en1 inet dhcp',
                                     'defs/my/std.bak': 'some_bogus dontreadme'})[0]
-        self.assertEqual(yaml.load(out), {'network': {
+        self.assertEqual(_load_yaml(out), {'network': {
             'version': 2,
             'ethernets': {'en1': {'dhcp4': True}}}}, out.decode())
 
@@ -240,7 +248,7 @@ source-directory /etc/network/interfaces.d''')[0]
         out = self.do_test('iface lo inet loopback\nauto lo\nsource interfaces.d/*.cfg',
                            dropins={'interfaces.d/std.cfg': 'auto en1\niface en1 inet dhcp',
                                     'interfaces.d/std.cfgold': 'some_bogus dontreadme'})[0]
-        self.assertEqual(yaml.load(out), {'network': {
+        self.assertEqual(_load_yaml(out), {'network': {
             'version': 2,
             'ethernets': {'en1': {'dhcp4': True}}}}, out.decode())
 
@@ -248,21 +256,21 @@ source-directory /etc/network/interfaces.d''')[0]
         out = self.do_test('iface lo inet loopback\nauto lo\nsource /etc/network/*.cfg',
                            dropins={'std.cfg': 'auto en1\niface en1 inet dhcp',
                                     'std.cfgold': 'some_bogus dontreadme'})[0]
-        self.assertEqual(yaml.load(out), {'network': {
+        self.assertEqual(_load_yaml(out), {'network': {
             'version': 2,
             'ethernets': {'en1': {'dhcp4': True}}}}, out.decode())
 
     def test_allow(self):
         out = self.do_test('allow-hotplug en1\niface en1 inet dhcp\n'
                            'allow-auto en2\niface en2 inet dhcp')[0]
-        self.assertEqual(yaml.load(out), {'network': {
+        self.assertEqual(_load_yaml(out), {'network': {
             'version': 2,
             'ethernets': {'en1': {'dhcp4': True},
                           'en2': {'dhcp4': True}}}}, out.decode())
 
     def test_no_scripts(self):
         out = self.do_test('auto en1\niface en1 inet dhcp\nno-scripts en1')[0]
-        self.assertEqual(yaml.load(out), {'network': {
+        self.assertEqual(_load_yaml(out), {'network': {
             'version': 2,
             'ethernets': {'en1': {'dhcp4': True}}}}, out.decode())
 
@@ -276,7 +284,7 @@ source-directory /etc/network/interfaces.d''')[0]
     def test_write_file_haveconfig(self):
         (out, err) = self.do_test('auto en1\niface en1 inet dhcp', dry_run=False)
         with open(self.converted_path) as f:
-            config = yaml.load(f)
+            config = _load_yaml(f)
         self.assertEqual(config, {'network': {
             'version': 2,
             'ethernets': {'en1': {'dhcp4': True}}}})
@@ -302,13 +310,13 @@ source-directory /etc/network/interfaces.d''')[0]
 
     def test_static_ipv4_prefix(self):
         out = self.do_test('auto en1\niface en1 inet static\naddress 1.2.3.4/8', dry_run=True)[0]
-        self.assertEqual(yaml.load(out), {'network': {
+        self.assertEqual(_load_yaml(out), {'network': {
             'version': 2,
             'ethernets': {'en1': {'addresses': ["1.2.3.4/8"]}}}}, out.decode())
 
     def test_static_ipv4_netmask(self):
         out = self.do_test('auto en1\niface en1 inet static\naddress 1.2.3.4\nnetmask 255.0.0.0', dry_run=True)[0]
-        self.assertEqual(yaml.load(out), {'network': {
+        self.assertEqual(_load_yaml(out), {'network': {
             'version': 2,
             'ethernets': {'en1': {'addresses': ["1.2.3.4/8"]}}}}, out.decode())
 
@@ -349,14 +357,14 @@ source-directory /etc/network/interfaces.d''')[0]
 
     def test_static_ipv6_prefix(self):
         out = self.do_test('auto en1\niface en1 inet6 static\naddress fc00:0123:4567:89ab:cdef::1234/64', dry_run=True)[0]
-        self.assertEqual(yaml.load(out), {'network': {
+        self.assertEqual(_load_yaml(out), {'network': {
             'version': 2,
             'ethernets': {'en1': {'addresses': ["fc00:123:4567:89ab:cdef::1234/64"]}}}}, out.decode())
 
     def test_static_ipv6_netmask(self):
         out = self.do_test('auto en1\niface en1 inet6 static\n'
                            'address fc00:0123:4567:89ab:cdef::1234\nnetmask 64', dry_run=True)[0]
-        self.assertEqual(yaml.load(out), {'network': {
+        self.assertEqual(_load_yaml(out), {'network': {
             'version': 2,
             'ethernets': {'en1': {'addresses': ["fc00:123:4567:89ab:cdef::1234/64"]}}}}, out.decode())
 
@@ -404,7 +412,7 @@ source-directory /etc/network/interfaces.d''')[0]
     def test_static_ipv6_accept_ra_0(self):
         out = self.do_test('auto en1\niface en1 inet6 static\n'
                            'address fc00:0123:4567:89ab:cdef::1234/64\naccept_ra 0', dry_run=True)[0]
-        self.assertEqual(yaml.load(out), {'network': {
+        self.assertEqual(_load_yaml(out), {'network': {
             'version': 2,
             'ethernets': {'en1': {'addresses': ["fc00:123:4567:89ab:cdef::1234/64"],
                                   'accept_ra': False}}}}, out.decode())
@@ -412,7 +420,7 @@ source-directory /etc/network/interfaces.d''')[0]
     def test_static_ipv6_accept_ra_1(self):
         out = self.do_test('auto en1\niface en1 inet6 static\n'
                            'address fc00:0123:4567:89ab:cdef::1234/64\naccept_ra 1', dry_run=True)[0]
-        self.assertEqual(yaml.load(out), {'network': {
+        self.assertEqual(_load_yaml(out), {'network': {
             'version': 2,
             'ethernets': {'en1': {'addresses': ["fc00:123:4567:89ab:cdef::1234/64"],
                                   'accept_ra': True}}}}, out.decode())
@@ -438,7 +446,7 @@ iface en1 inet static
 iface en1 inet6 static
   address fc00:0123:4567:89ab:cdef::1234/64
   gateway fc00:0123:4567:89ab::1""", dry_run=True)[0]
-        self.assertEqual(yaml.load(out), {'network': {
+        self.assertEqual(_load_yaml(out), {'network': {
             'version': 2,
             'ethernets': {'en1':
                           {'addresses': ["1.2.3.4/8", "fc00:123:4567:89ab:cdef::1234/64"],
@@ -455,7 +463,7 @@ iface en1 inet static
 iface en1 inet6 static
   address fc00:0123:4567:89ab:cdef::1234/64
   dns-nameservers fc00:0123:4567:89ab:1::1  fc00:0123:4567:89ab:2::1""", dry_run=True)[0]
-        self.assertEqual(yaml.load(out), {'network': {
+        self.assertEqual(_load_yaml(out), {'network': {
             'version': 2,
             'ethernets': {'en1':
                           {'addresses': ["1.2.3.4/8", "fc00:123:4567:89ab:cdef::1234/64"],
@@ -467,7 +475,7 @@ iface en1 inet6 static
 
     def test_static_dns2(self):
         out = self.do_test('auto en1\niface en1 inet static\naddress 1.2.3.4/8\ndns-search foo  foo.bar', dry_run=True)[0]
-        self.assertEqual(yaml.load(out), {'network': {
+        self.assertEqual(_load_yaml(out), {'network': {
             'version': 2,
             'ethernets': {'en1': {'addresses': ["1.2.3.4/8"],
                                   'nameservers': {
@@ -476,7 +484,7 @@ iface en1 inet6 static
 
     def test_static_mtu(self):
         out = self.do_test('auto en1\niface en1 inet static\naddress 1.2.3.4/8\nmtu 1280', dry_run=True)[0]
-        self.assertEqual(yaml.load(out), {'network': {
+        self.assertEqual(_load_yaml(out), {'network': {
             'version': 2,
             'ethernets': {'en1': {'addresses': ["1.2.3.4/8"],
                                   'mtu': 1280}}}}, out.decode())
@@ -494,7 +502,7 @@ iface en1 inet6 static
 
     def test_static_hwaddress(self):
         out = self.do_test('auto en1\niface en1 inet static\naddress 1.2.3.4/8\nhwaddress 52:54:00:6b:3c:59', dry_run=True)[0]
-        self.assertEqual(yaml.load(out), {'network': {
+        self.assertEqual(_load_yaml(out), {'network': {
             'version': 2,
             'ethernets': {'en1': {'addresses': ["1.2.3.4/8"],
                                   'macaddress': '52:54:00:6b:3c:59'}}}}, out.decode())
