@@ -19,7 +19,7 @@
 import os
 import stat
 
-from .base import TestBase, ND_DHCP4, ND_WIFI_DHCP4
+from .base import TestBase, ND_DHCP4, ND_WIFI_DHCP4, SD_WPA
 
 
 class TestNetworkd(TestBase):
@@ -225,6 +225,10 @@ network={
             self.assertEqual(stat.S_IMODE(os.fstat(f.fileno()).st_mode), 0o600)
         self.assertTrue(os.path.isfile(os.path.join(
             self.workdir.name, 'run/systemd/system/netplan-wpa-eth0.service')))
+
+        with open(os.path.join(self.workdir.name, 'run/systemd/system/netplan-wpa-eth0.service')) as f:
+            self.assertEqual(f.read(), SD_WPA % {'iface': 'eth0', 'drivers': 'wired'})
+            self.assertEqual(stat.S_IMODE(os.fstat(f.fileno()).st_mode), 0o644)
         self.assertTrue(os.path.islink(os.path.join(
             self.workdir.name, 'run/systemd/system/systemd-networkd.service.wants/netplan-wpa-eth0.service')))
 
@@ -291,9 +295,6 @@ id=netplan-wl0-Joe's Home
 type=wifi
 interface-name=wl0
 
-[ethernet]
-wake-on-lan=0
-
 [ipv4]
 method=auto
 
@@ -313,9 +314,6 @@ id=netplan-wl0-Luke's Home
 type=wifi
 interface-name=wl0
 
-[ethernet]
-wake-on-lan=0
-
 [ipv4]
 method=auto
 
@@ -335,9 +333,6 @@ id=netplan-wl0-workplace
 type=wifi
 interface-name=wl0
 
-[ethernet]
-wake-on-lan=0
-
 [ipv4]
 method=auto
 
@@ -362,9 +357,6 @@ id=netplan-wl0-workplace2
 type=wifi
 interface-name=wl0
 
-[ethernet]
-wake-on-lan=0
-
 [ipv4]
 method=auto
 
@@ -389,9 +381,6 @@ id=netplan-wl0-workplacehashed
 type=wifi
 interface-name=wl0
 
-[ethernet]
-wake-on-lan=0
-
 [ipv4]
 method=auto
 
@@ -416,9 +405,6 @@ id=netplan-wl0-customernet
 type=wifi
 interface-name=wl0
 
-[ethernet]
-wake-on-lan=0
-
 [ipv4]
 method=auto
 
@@ -447,9 +433,6 @@ id=netplan-wl0-opennet
 type=wifi
 interface-name=wl0
 
-[ethernet]
-wake-on-lan=0
-
 [ipv4]
 method=auto
 
@@ -465,9 +448,6 @@ id=netplan-wl0-peer2peer
 type=wifi
 interface-name=wl0
 
-[ethernet]
-wake-on-lan=0
-
 [ipv4]
 method=auto
 
@@ -577,3 +557,14 @@ class TestConfigErrors(TestBase):
           password: "LoremipsumdolorsitametconsecteturadipiscingelitCrastemporvelitnu"
       dhcp4: yes''', expect_fail=True)
         self.assertIn("PSK length of 64 is only supported for hex-digit representation", err)
+
+    def test_auth_networkd_wire_psk_64_non_hexdigit(self):
+        err = self.generate('''network:
+  version: 2
+  ethernets:
+    eth0:
+      auth:
+        key-management: psk
+        password: "LoremipsumdolorsitametconsecteturadipiscingelitCrastemporvelitnu"
+      dhcp4: yes''', expect_fail=True)
+        self.assertIn("PSK length of 64 is only supported for hex-digit representation", err)
