@@ -69,6 +69,7 @@ method=auto
 [ipv6]
 dns-search=
 method=auto
+ip6-privacy=0
 '''.format(UUID))
         self.assert_netplan({UUID: '''network:
   version: 2
@@ -92,6 +93,8 @@ method=auto
         name: "T-Mobile Funkadelic 2"
         passthrough:
           gsm.home-only: "true"
+          ipv4.dns-search: ""
+          ipv6.dns-search: ""
 '''.format(UUID, UUID)})
 
     def test_keyfile_cdma(self):
@@ -200,9 +203,9 @@ route-metric=4242
 
 [ipv6]
 addr-gen-mode=eui64
-token=1234::3
 dns-search=
 method=auto
+ip6-privacy=0
 ignore-auto-routes=true
 never-default=true
 route-metric=4242
@@ -225,15 +228,35 @@ route-metric=4242
         route-metric: 4242
       macaddress: "00:11:22:33:44:55"
       ipv6-address-generation: "eui64"
-      ipv6-address-token: "1234::3"
       mtu: 1500
       networkmanager:
         uuid: "{}"
         name: "Test"
         passthrough:
+          ipv4.dns-search: ""
+          ipv6.dns-search: ""
           proxy._: ""
 '''.format(UUID, UUID)})
 
+    def test_keyfile_fail_validation(self):
+        err = self.generate_from_keyfile('''[connection]
+id=Test
+uuid={}
+type=ethernet
+
+[ethernet]
+wake-on-lan=0
+
+[ipv4]
+method=auto
+
+[ipv6]
+addr-gen-mode=eui64
+token=::42
+method=auto
+'''.format(UUID), expect_fail=True)
+        self.assertIn('Error in network definition:', err)
+
     def test_keyfile_method_manual(self):
         self.generate_from_keyfile('''[connection]
 id=Test
@@ -259,6 +282,7 @@ addr-gen-mode=stable-privacy
 dns-search=bar.local
 dns=dead:beef::2;
 method=manual
+ip6-privacy=2
 address1=1:2:3::9/128
 gateway=6:6::6
 route1=dead:beef::1/128,2001:1234::2
@@ -289,13 +313,14 @@ route1_options=unknown=invalid,
       gateway4: 6.6.6.6
       gateway6: 6:6::6
       ipv6-address-generation: "stable-privacy"
+      ipv6-privacy: true
       routes:
       - metric: 42
         table: 102
         mtu: 1024
         congestion-window: 44
         advertised-receive-window: 33
-        on-link: "true"
+        on-link: true
         from: "10.10.10.11"
         to: "1.1.2.2/16"
         via: "8.8.8.8"
@@ -308,8 +333,10 @@ route1_options=unknown=invalid,
         uuid: "{}"
         name: "Test"
         passthrough:
+          ipv4.dns-search: "foo.local;bar.remote;"
           ipv4.method: "manual"
           ipv4.address1: "1.2.3.4/24,8.8.8.8"
+          ipv6.dns-search: "bar.local"
           ipv6.route1: "dead:beef::1/128,2001:1234::2"
           ipv6.route1_options: "unknown=invalid,"
           proxy._: ""
@@ -354,13 +381,13 @@ route1_options=unknown=invalid,
         self._template_keyfile_type('bonds', 'bond')
 
     def test_keyfile_type_vlan(self):
-        self._template_keyfile_type('vlans', 'vlan')
+        self._template_keyfile_type('nm-devices', 'vlan', False)
 
     def test_keyfile_type_tunnel(self):
-        self._template_keyfile_type('tunnels', 'ip-tunnel', False)
+        self._template_keyfile_type('nm-devices', 'ip-tunnel', False)
 
     def test_keyfile_type_wireguard(self):
-        self._template_keyfile_type('tunnels', 'wireguard', False)
+        self._template_keyfile_type('nm-devices', 'wireguard', False)
 
     def test_keyfile_type_other(self):
         self._template_keyfile_type('nm-devices', 'dummy', False)
@@ -412,6 +439,7 @@ dns-search='''.format(UUID))
             name: "myid with spaces"
             passthrough:
               connection.permissions: ""
+              ipv4.dns-search: ""
       networkmanager:
         uuid: "{}"
         name: "myid with spaces"
@@ -472,6 +500,7 @@ dns-search='''.format(UUID, method))
             name: "testnet"
             passthrough:
               connection.permissions: ""
+              ipv4.dns-search: ""
       networkmanager:
         uuid: "{}"
         name: "testnet"
@@ -685,6 +714,7 @@ dns-search=
 method=ignore
 addr-gen-mode=1
 dns-search=
+ip6-privacy=0
 
 [wifi]
 ssid=my-hotspot
@@ -718,7 +748,9 @@ psk=test1234
             passthrough:
               connection.autoconnect: "false"
               connection.permissions: ""
+              ipv4.dns-search: ""
               ipv6.addr-gen-mode: "1"
+              ipv6.dns-search: ""
               wifi.mac-address-blacklist: ""
               wifi-security.group: "ccmp;"
               wifi-security.pairwise: "ccmp;"
@@ -774,21 +806,24 @@ address1=1.2.3.4/24
 
 [ipv6]
 method=ignore
-'''.format(UUID), netdef_id='enblue', expect_fail=False, filename="some.keyfile")
+'''.format(UUID))
         self.assert_netplan({UUID: '''network:
   version: 2
-  vlans:
-    enblue:
+  nm-devices:
+    NM-{}:
       renderer: NetworkManager
-      addresses:
-      - "1.2.3.4/24"
-      id: 1
       networkmanager:
         uuid: "{}"
         name: "netplan-enblue"
         passthrough:
+          connection.type: "vlan"
+          connection.interface-name: "enblue"
+          vlan.id: "1"
           vlan.parent: "en1"
-'''.format(UUID)})
+          ipv4.method: "manual"
+          ipv4.address1: "1.2.3.4/24"
+          ipv6.method: "ignore"
+'''.format(UUID, UUID)})
 
     def test_keyfile_bridge(self):
         self.generate_from_keyfile('''[connection]
@@ -892,7 +927,7 @@ method=auto
 
 [ipv6]
 method=ignore
-'''.format(UUID), netdef_id='bn0')
+'''.format(UUID), netdef_id='bn0', expect_fail=False, filename='some.keyfile')
         self.assert_netplan({UUID: '''network:
   version: 2
   bonds:
@@ -988,6 +1023,7 @@ dns=8.8.8.8;8.8.4.4;8.8.8.8;8.8.4.4;8.8.8.8;8.8.4.4;
 [ipv6]
 method=auto
 addr-gen-mode=1
+ip6-privacy=0
 '''.format(UUID))
         self.assert_netplan({UUID: '''network:
   version: 2
@@ -1016,3 +1052,193 @@ addr-gen-mode=1
           ipv4.address3: "10.10.246.132/24"
           ipv6.addr-gen-mode: "1"
 '''.format(UUID, UUID)})
+
+    def test_keyfile_netplan0103_compat(self):
+        self.generate_from_keyfile('''[connection]
+id=Work Wired
+uuid={}
+type=ethernet
+autoconnect=false
+permissions=
+timestamp=305419896
+
+[ethernet]
+mac-address=99:88:77:66:55:44
+mac-address-blacklist=
+mtu=900
+
+[ipv4]
+address1=192.168.0.5/24,192.168.0.1
+address2=1.2.3.4/8
+dns=4.2.2.1;4.2.2.2;
+dns-search=
+method=manual
+route1=10.10.10.2/24,10.10.10.1,3
+route2=1.1.1.1/8,1.2.1.1,1
+route3=2.2.2.2/7
+route4=3.3.3.3/6,0.0.0.0,4
+route4_options=cwnd=10,mtu=1492,src=1.2.3.4
+
+[ipv6]
+addr-gen-mode=stable-privacy
+address1=abcd::beef/64
+address2=dcba::beef/56
+dns=1::cafe;2::cafe;
+dns-search=wallaceandgromit.com;
+method=manual
+ip6-privacy=1
+route1=1:2:3:4:5:6:7:8/64,8:7:6:5:4:3:2:1,3
+route2=2001::1000/56,2001::1111,1
+route3=4:5:6:7:8:9:0:1/63,::,5
+route4=5:6:7:8:9:0:1:2/62
+
+[proxy]
+
+
+'''.format(UUID))
+        self.assert_netplan({UUID: '''network:
+  version: 2
+  ethernets:
+    NM-{}:
+      renderer: NetworkManager
+      match:
+        macaddress: "99:88:77:66:55:44"
+      addresses:
+      - "192.168.0.5/24"
+      - "1.2.3.4/8"
+      - "abcd::beef/64"
+      - "dcba::beef/56"
+      nameservers:
+        addresses:
+        - 4.2.2.1
+        - 4.2.2.2
+        - 1::cafe
+        - 2::cafe
+        search:
+        - wallaceandgromit.com
+      ipv6-address-generation: "stable-privacy"
+      mtu: 900
+      routes:
+      - metric: 3
+        to: "10.10.10.2/24"
+        via: "10.10.10.1"
+      - metric: 1
+        to: "1.1.1.1/8"
+        via: "1.2.1.1"
+      - scope: "link"
+        to: "2.2.2.2/7"
+      - scope: "link"
+        metric: 4
+        mtu: 1492
+        from: "1.2.3.4"
+        to: "3.3.3.3/6"
+      - metric: 3
+        to: "1:2:3:4:5:6:7:8/64"
+        via: "8:7:6:5:4:3:2:1"
+      - metric: 1
+        to: "2001::1000/56"
+        via: "2001::1111"
+      - scope: "link"
+        metric: 5
+        to: "4:5:6:7:8:9:0:1/63"
+      - scope: "link"
+        to: "5:6:7:8:9:0:1:2/62"
+      wakeonlan: true
+      networkmanager:
+        uuid: "{}"
+        name: "Work Wired"
+        passthrough:
+          connection.autoconnect: "false"
+          connection.permissions: ""
+          connection.timestamp: "305419896"
+          ethernet.mac-address-blacklist: ""
+          ipv4.address1: "192.168.0.5/24,192.168.0.1"
+          ipv4.dns-search: ""
+          ipv4.method: "manual"
+          ipv4.route4: "3.3.3.3/6,0.0.0.0,4"
+          ipv4.route4_options: "cwnd=10,mtu=1492,src=1.2.3.4"
+          ipv6.dns-search: "wallaceandgromit.com;"
+          ipv6.ip6-privacy: "1"
+          proxy._: ""
+'''.format(UUID, UUID)})
+
+    def test_keyfile_tunnel_regression_lp1952967(self):
+        self.generate_from_keyfile('''[connection]
+id=IP tunnel connection 1
+uuid={}
+type=ip-tunnel
+autoconnect=false
+interface-name=gre10
+permissions=
+
+[ip-tunnel]
+local=10.20.20.1
+mode=2
+remote=10.20.20.2
+
+[ipv4]
+dns-search=
+method=auto
+
+[ipv6]
+addr-gen-mode=stable-privacy
+dns-search=
+method=auto
+
+[proxy]
+'''.format(UUID))
+        self.assert_netplan({UUID: '''network:
+  version: 2
+  nm-devices:
+    NM-{}:
+      renderer: NetworkManager
+      networkmanager:
+        uuid: "{}"
+        name: "IP tunnel connection 1"
+        passthrough:
+          connection.type: "ip-tunnel"
+          connection.autoconnect: "false"
+          connection.interface-name: "gre10"
+          connection.permissions: ""
+          ip-tunnel.local: "10.20.20.1"
+          ip-tunnel.mode: "2"
+          ip-tunnel.remote: "10.20.20.2"
+          ipv4.dns-search: ""
+          ipv4.method: "auto"
+          ipv6.addr-gen-mode: "stable-privacy"
+          ipv6.dns-search: ""
+          ipv6.method: "auto"
+          proxy._: ""
+'''.format(UUID, UUID)})
+
+    def test_keyfile_ip6_privacy_default_netplan_0104_compat(self):
+        self.generate_from_keyfile('''[connection]
+id=Test
+uuid={}
+type=ethernet
+
+[ethernet]
+mac-address=99:88:77:66:55:44
+
+[ipv4]
+method=auto
+
+[ipv6]
+method=auto
+'''.format(UUID))
+        self.assert_netplan({UUID: '''network:
+  version: 2
+  ethernets:
+    NM-{}:
+      renderer: NetworkManager
+      match:
+        macaddress: "99:88:77:66:55:44"
+      dhcp4: true
+      dhcp6: true
+      wakeonlan: true
+      networkmanager:
+        uuid: "{}"
+        name: "Test"
+        passthrough:
+          ipv6.ip6-privacy: "-1"
+'''.format(UUID, UUID)})
